From d148d7cb1fc11106426d9741d784cb60dcddb7c3 Mon Sep 17 00:00:00 2001
From: Nils Knappmeier <github@knappi.org>
Date: Tue, 24 Sep 2019 08:36:46 +0200
Subject: [PATCH 1/8] check "allowCallsToHelperMissing"

---
 types/test.ts | 4 +++-
 1 file changed, 3 insertions(+), 1 deletion(-)

diff --git a/types/test.ts b/types/test.ts
index 341a3c2f9..9ecc2d167 100644
--- a/types/test.ts
+++ b/types/test.ts
@@ -97,7 +97,7 @@ Handlebars.VM.resolvePartial = <T>(partial: HandlebarsTemplateDelegate<T> | unde
   // transform name.
   options.name = name;
   return originalResolvePartial(partial, context, options);
-}
+};
 
 
 // #1544, allow custom helpers in knownHelpers
@@ -107,3 +107,5 @@ Handlebars.compile('test', {
     customHelper: true
   }
 });
+
+Handlebars.compile('test')({},{allowCallsToHelperMissing: true});

From 93444c58db259bb1e5073d4e815a5aab65440e33 Mon Sep 17 00:00:00 2001
From: Matt Hinchliffe <i-like-robots@users.noreply.github.com>
Date: Tue, 24 Sep 2019 14:02:55 +0100
Subject: [PATCH 2/8] Ensure allowCallsToHelperMissing runtime option is
 optional in typings

---
 types/index.d.ts | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/types/index.d.ts b/types/index.d.ts
index bf8d17564..a1b1693e9 100644
--- a/types/index.d.ts
+++ b/types/index.d.ts
@@ -29,7 +29,7 @@ declare namespace Handlebars {
       decorators?: { [name: string]: Function };
       data?: any;
       blockParams?: any[];
-      allowCallsToHelperMissing: boolean;
+      allowCallsToHelperMissing?: boolean;
   }
 
   export interface HelperOptions {

From 64ecb9ea84f691bac1427879c5088d01b46b4c7a Mon Sep 17 00:00:00 2001
From: Nils Knappmeier <github@knappi.org>
Date: Tue, 24 Sep 2019 22:28:52 +0200
Subject: [PATCH 3/8] add test for #1560

---
 types/test.ts | 2 ++
 1 file changed, 2 insertions(+)

diff --git a/types/test.ts b/types/test.ts
index 9ecc2d167..72d6ff70a 100644
--- a/types/test.ts
+++ b/types/test.ts
@@ -109,3 +109,5 @@ Handlebars.compile('test', {
 });
 
 Handlebars.compile('test')({},{allowCallsToHelperMissing: true});
+
+Handlebars.compile('test')({},{});
\ No newline at end of file

From 12668388294ee8bb1c07c0d9d5c6ee083910f3a5 Mon Sep 17 00:00:00 2001
From: Nils Knappmeier <github@knappi.org>
Date: Tue, 24 Sep 2019 23:45:09 +0200
Subject: [PATCH 4/8] do not break on precompiled templates from Handlebars
 >=4.0.0 <4.3.0

- The version-range above have compiler version 7 and
  precompiled templates expecte the (block)
  HelperMissing-functions in "helpers" and not in "container.hooks".
- Handlebars now accepts precompiled templates of version 7.
- If a precompiled template with version 7 is loaded,
  the (block)HelperMissing-functions are kept in "helpers"
---
 lib/handlebars/base.js    |  1 +
 lib/handlebars/runtime.js | 36 +++++++++++++++++++++---------------
 spec/regressions.js       | 32 ++++++++++++++++++++++++++++++++
 spec/runtime.js           |  2 +-
 4 files changed, 55 insertions(+), 16 deletions(-)

diff --git a/lib/handlebars/base.js b/lib/handlebars/base.js
index 2703dafd6..a327fc6c7 100644
--- a/lib/handlebars/base.js
+++ b/lib/handlebars/base.js
@@ -6,6 +6,7 @@ import logger from './logger';
 
 export const VERSION = '4.3.0';
 export const COMPILER_REVISION = 8;
+export const LAST_COMPATIBLE_COMPILER_REVISION = 7;
 
 export const REVISION_CHANGES = {
   1: '<= 1.0.rc.2', // 1.0.rc.2 is actually rev2 but doesn't report it
diff --git a/lib/handlebars/runtime.js b/lib/handlebars/runtime.js
index ce4630463..763016de7 100644
--- a/lib/handlebars/runtime.js
+++ b/lib/handlebars/runtime.js
@@ -1,23 +1,25 @@
 import * as Utils from './utils';
 import Exception from './exception';
-import {COMPILER_REVISION, createFrame, REVISION_CHANGES} from './base';
+import {COMPILER_REVISION, createFrame, LAST_COMPATIBLE_COMPILER_REVISION, REVISION_CHANGES} from './base';
 import {moveHelperToHooks} from './helpers';
 
 export function checkRevision(compilerInfo) {
   const compilerRevision = compilerInfo && compilerInfo[0] || 1,
         currentRevision = COMPILER_REVISION;
 
-  if (compilerRevision !== currentRevision) {
-    if (compilerRevision < currentRevision) {
-      const runtimeVersions = REVISION_CHANGES[currentRevision],
-            compilerVersions = REVISION_CHANGES[compilerRevision];
-      throw new Exception('Template was precompiled with an older version of Handlebars than the current runtime. ' +
-            'Please update your precompiler to a newer version (' + runtimeVersions + ') or downgrade your runtime to an older version (' + compilerVersions + ').');
-    } else {
-      // Use the embedded version info since the runtime doesn't know about this revision yet
-      throw new Exception('Template was precompiled with a newer version of Handlebars than the current runtime. ' +
-            'Please update your runtime to a newer version (' + compilerInfo[1] + ').');
-    }
+  if (compilerRevision >= LAST_COMPATIBLE_COMPILER_REVISION && compilerRevision <= COMPILER_REVISION) {
+    return;
+  }
+
+  if (compilerRevision < LAST_COMPATIBLE_COMPILER_REVISION) {
+    const runtimeVersions = REVISION_CHANGES[currentRevision],
+          compilerVersions = REVISION_CHANGES[compilerRevision];
+    throw new Exception('Template was precompiled with an older version of Handlebars than the current runtime. ' +
+          'Please update your precompiler to a newer version (' + runtimeVersions + ') or downgrade your runtime to an older version (' + compilerVersions + ').');
+  } else {
+    // Use the embedded version info since the runtime doesn't know about this revision yet
+    throw new Exception('Template was precompiled with a newer version of Handlebars than the current runtime. ' +
+          'Please update your runtime to a newer version (' + compilerInfo[1] + ').');
   }
 }
 
@@ -37,6 +39,9 @@ export function template(templateSpec, env) {
   // for external users to override these as pseudo-supported APIs.
   env.VM.checkRevision(templateSpec.compiler);
 
+  // backwards compatibility for precompiled templates with compiler-version 7 (<4.3.0)
+  const templateWasPrecompiledWithCompilerV7 = templateSpec.compiler && templateSpec.compiler[0] === 7;
+
   function invokePartialWrapper(partial, context, options) {
     if (options.hash) {
       context = Utils.extend({}, context, options.hash);
@@ -163,9 +168,10 @@ export function template(templateSpec, env) {
       }
 
       container.hooks = {};
-      let keepHelper = options.allowCallsToHelperMissing;
-      moveHelperToHooks(container, 'helperMissing', keepHelper);
-      moveHelperToHooks(container, 'blockHelperMissing', keepHelper);
+
+      let keepHelperInHelpers = options.allowCallsToHelperMissing || templateWasPrecompiledWithCompilerV7;
+      moveHelperToHooks(container, 'helperMissing', keepHelperInHelpers);
+      moveHelperToHooks(container, 'blockHelperMissing', keepHelperInHelpers);
 
     } else {
       container.helpers = options.helpers;
diff --git a/spec/regressions.js b/spec/regressions.js
index 1678d80c1..a06360c2b 100644
--- a/spec/regressions.js
+++ b/spec/regressions.js
@@ -291,4 +291,36 @@ describe('Regressions', function() {
     };
     shouldCompileToWithPartials(string, [{}, {}, partials], true, 'template  block  partial  block  template');
   });
+
+  describe('GH-1561: 4.3.x should still work with precompiled templates from 4.0.0 <= x < 4.3.0', function() {
+
+    it('should compile and execute templates', function() {
+      var newHandlebarsInstance = Handlebars.create();
+
+      registerTemplate(newHandlebarsInstance);
+      newHandlebarsInstance.registerHelper('loud', function(value) {
+        return value.toUpperCase();
+      });
+      let result = newHandlebarsInstance.templates['test.hbs']({name: 'yehuda'});
+      equals(result.trim(), 'YEHUDA');
+    });
+
+    it('should call "helperMissing" if a helper is missing', function() {
+      var newHandlebarsInstance = Handlebars.create();
+
+      shouldThrow(() => {
+        registerTemplate(newHandlebarsInstance);
+        newHandlebarsInstance.templates['test.hbs']({});
+      }, Handlebars.Exception, 'Missing helper: "loud"');
+    });
+
+    // This is a only slightly modified precompiled templated from compiled with 4.2.1
+    function registerTemplate(Handlebars) {
+      var template = Handlebars.template, templates = Handlebars.templates = Handlebars.templates || {};
+      templates['test.hbs'] = template({'compiler': [7, '>= 4.0.0'], 'main': function(container, depth0, helpers, partials, data) {
+          return container.escapeExpression((helpers.loud || (depth0 && depth0.loud) || helpers.helperMissing).call(depth0 != null ? depth0 : (container.nullContext || {}), (depth0 != null ? depth0.name : depth0), {'name': 'loud', 'hash': {}, 'data': data}))
+              + '\n\n';
+        }, 'useData': true});
+    }
+  });
 });
diff --git a/spec/runtime.js b/spec/runtime.js
index a4830ad0c..3549640b7 100644
--- a/spec/runtime.js
+++ b/spec/runtime.js
@@ -21,7 +21,7 @@ describe('runtime', function() {
       shouldThrow(function() {
         Handlebars.template({
           main: {},
-          compiler: [Handlebars.COMPILER_REVISION - 1]
+          compiler: [Handlebars.LAST_COMPATIBLE_COMPILER_REVISION - 1]
         });
       }, Error, /Template was precompiled with an older version of Handlebars than the current runtime/);
       shouldThrow(function() {

From c2a17c7b30009059743e3cb3c677d0adc2002775 Mon Sep 17 00:00:00 2001
From: Nils Knappmeier <github@knappi.org>
Date: Wed, 25 Sep 2019 00:00:51 +0200
Subject: [PATCH 5/8] fix saucelabs tests (internet explorer)

---
 spec/regressions.js | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/spec/regressions.js b/spec/regressions.js
index a06360c2b..99042b5a2 100644
--- a/spec/regressions.js
+++ b/spec/regressions.js
@@ -308,7 +308,7 @@ describe('Regressions', function() {
     it('should call "helperMissing" if a helper is missing', function() {
       var newHandlebarsInstance = Handlebars.create();
 
-      shouldThrow(() => {
+      shouldThrow(function() {
         registerTemplate(newHandlebarsInstance);
         newHandlebarsInstance.templates['test.hbs']({});
       }, Handlebars.Exception, 'Missing helper: "loud"');

From e3639e240755ef713e128e7c67d30443ea01b732 Mon Sep 17 00:00:00 2001
From: Nils Knappmeier <github@knappi.org>
Date: Wed, 25 Sep 2019 00:18:02 +0200
Subject: [PATCH 6/8] fix saucelabs tests (internet explorer)

---
 spec/regressions.js | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/spec/regressions.js b/spec/regressions.js
index 99042b5a2..58db65bcd 100644
--- a/spec/regressions.js
+++ b/spec/regressions.js
@@ -301,7 +301,7 @@ describe('Regressions', function() {
       newHandlebarsInstance.registerHelper('loud', function(value) {
         return value.toUpperCase();
       });
-      let result = newHandlebarsInstance.templates['test.hbs']({name: 'yehuda'});
+      var result = newHandlebarsInstance.templates['test.hbs']({name: 'yehuda'});
       equals(result.trim(), 'YEHUDA');
     });
 

From 2d5579078b36734b7db7f65b24a1ce5340164f78 Mon Sep 17 00:00:00 2001
From: Nils Knappmeier <github@knappi.org>
Date: Wed, 25 Sep 2019 00:31:48 +0200
Subject: [PATCH 7/8] Update release notes

---
 release-notes.md | 12 +++++++++++-
 1 file changed, 11 insertions(+), 1 deletion(-)

diff --git a/release-notes.md b/release-notes.md
index 8d11f07ba..45c90a162 100644
--- a/release-notes.md
+++ b/release-notes.md
@@ -2,7 +2,17 @@
 
 ## Development
 
-[Commits](https://github.com/wycats/handlebars.js/compare/v4.3.0...master)
+[Commits](https://github.com/wycats/handlebars.js/compare/v4.3.1...master)
+
+## v4.3.1 - September 25th, 2019
+Fixes:
+
+- do not break on precompiled templates from Handlebars >=4.0.0 <4.3.0 - 1266838, #1561
+- Ensure allowCallsToHelperMissing runtime option is optional in typings - 93444c5, 64ecb9e, #1560
+
+
+
+[Commits](https://github.com/wycats/handlebars.js/compare/v4.3.0...v4.3.1)
 
 ## v4.3.0 - September 24th, 2019
 Fixes:

From 050cca0866b3496852d9b198141807c32e4dae81 Mon Sep 17 00:00:00 2001
From: Nils Knappmeier <github@knappi.org>
Date: Wed, 25 Sep 2019 00:32:32 +0200
Subject: [PATCH 8/8] v4.3.1

---
 components/bower.json           | 2 +-
 components/handlebars.js.nuspec | 2 +-
 components/package.json         | 2 +-
 lib/handlebars/base.js          | 2 +-
 package.json                    | 2 +-
 5 files changed, 5 insertions(+), 5 deletions(-)

diff --git a/components/bower.json b/components/bower.json
index 60e09e4d4..f26015d4c 100644
--- a/components/bower.json
+++ b/components/bower.json
@@ -1,6 +1,6 @@
 {
   "name": "handlebars",
-  "version": "4.3.0",
+  "version": "4.3.1",
   "main": "handlebars.js",
   "license": "MIT",
   "dependencies": {}
diff --git a/components/handlebars.js.nuspec b/components/handlebars.js.nuspec
index ff656b2ea..7f708a582 100644
--- a/components/handlebars.js.nuspec
+++ b/components/handlebars.js.nuspec
@@ -2,7 +2,7 @@
 <package>
 	<metadata>
 		<id>handlebars.js</id>
-		<version>4.3.0</version>
+		<version>4.3.1</version>
 		<authors>handlebars.js Authors</authors>
 		<licenseUrl>https://github.com/wycats/handlebars.js/blob/master/LICENSE</licenseUrl>
 		<projectUrl>https://github.com/wycats/handlebars.js/</projectUrl>
diff --git a/components/package.json b/components/package.json
index a8cfd290a..09278636f 100644
--- a/components/package.json
+++ b/components/package.json
@@ -1,6 +1,6 @@
 {
   "name": "handlebars",
-  "version": "4.3.0",
+  "version": "4.3.1",
   "license": "MIT",
   "jspm": {
     "main": "handlebars",
diff --git a/lib/handlebars/base.js b/lib/handlebars/base.js
index a327fc6c7..631f5b0f8 100644
--- a/lib/handlebars/base.js
+++ b/lib/handlebars/base.js
@@ -4,7 +4,7 @@ import {registerDefaultHelpers} from './helpers';
 import {registerDefaultDecorators} from './decorators';
 import logger from './logger';
 
-export const VERSION = '4.3.0';
+export const VERSION = '4.3.1';
 export const COMPILER_REVISION = 8;
 export const LAST_COMPATIBLE_COMPILER_REVISION = 7;
 
diff --git a/package.json b/package.json
index 6b56332cb..f616e4879 100644
--- a/package.json
+++ b/package.json
@@ -1,7 +1,7 @@
 {
   "name": "handlebars",
   "barename": "handlebars",
-  "version": "4.3.0",
+  "version": "4.3.1",
   "description": "Handlebars provides the power necessary to let you build semantic templates effectively with no frustration",
   "homepage": "http://www.handlebarsjs.com/",
   "keywords": [
